import 'package:flutter_test/flutter_test.dart';
import 'package:get_storage/src/storage_impl.dart';

void main() async {
  final g = GetStorage();
  var container1 = await GetStorage.init();

  g.erase();

  test('write, read listen, e removeListen', () async {
    String valueListen = "";
    String valueKeyListen = "";
    g.write('test', 'a');

    void boxListener() => valueListen = g.read('test');
    void boxKeyListener(k) => valueKeyListen = k;
    g.listen(boxListener);
    g.listenKey('test', boxKeyListener);
    expect({'test': 'a'}, g.listenable.value);
    expect('a', g.read('test'));
    await g.writeIfNull('test', 'add');
    expect('a', g.read('test'));
    g.writeIfNull('test2', 'add');
    expect('add', g.read('test2'));
    await g.write('test', 'b');
    expect('b', g.read<String>('test'));
    expect('b', valueListen);
    expect('b', valueKeyListen);

    g.removeListen(boxListener);
    g.removeKeyListen(boxKeyListener);

    await g.write('test', 'c');

    expect('c', g.read<String>('test'));
    expect('b', valueListen);
    await g.write('test', 'd');

    expect('d', g.read<String>('test'));
  });

  test('Write and read', () async {
    g.erase();
    final t1 = DateTime.now();
    var list = new List<int>.generate(50, (i) {
      int count = i + 1;
      return count;
    });

    for (int i in list) {
      await g.write('write', i);
    }

    final readSecs = DateTime.now().difference(t1).inMilliseconds * 0.000001;
    print(readSecs);

    expect(list.last, g.read('write'));
  });

  test('Write, read, remove and exists', () {
    g.erase();
    // expect(g.hasData('write'), false);

    var list = new List<int>.generate(50, (i) {
      int count = i + 1;
      g.write('write', count);
      print(count);
      return count;
    });
    expect(list.last, g.read('write'));
    g.remove('write');
    expect(null, g.read('write'));
  });

  test('newContainer', () async {
    await GetStorage.init('newContainer');
    final newContainer = GetStorage('newContainer');

    /// Attempting to start a Container that has already started must return the container already created.
    var container2 = await GetStorage.init();
    expect(container1 == container2, true);

    newContainer.write('test', '1234');
    g.write('test', 'a');
    expect(g.read('test') == newContainer.read('test'), false);
    g.erase();
  });
}
